package cn.ezhizao.project.business.controller;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

import cn.ezhizao.common.utils.ServletUtils;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import cn.ezhizao.project.business.domain.BizMeasurement;
import cn.ezhizao.project.business.service.IBizMeasurementService;

/**
 * 计量单位Controller
 *
 * @author ezhizao
 * @date 2025-04-23
 */
@Api("计量单位管理")
@RestController
@RequestMapping("/business/measurement")
public class BizMeasurementController extends BaseController
{
    @Resource
    private IBizMeasurementService bizMeasurementService;
    public static final String PAGE_SIZE = "pageSize";

    /**
     * 查询计量单位列表
     */
    @ApiOperation("获取计量单位详细")
    @GetMapping("/list")
    public TableDataInfo list(BizMeasurement bizMeasurement) throws NoSuchFieldException, IllegalAccessException
    {
        String pageSizeStr = ServletUtils.getParameter(PAGE_SIZE);
        if (pageSizeStr !=null) {
            startPage();
        }
        List<BizMeasurement> list = bizMeasurementService.getList(bizMeasurement);
        return getDataTable(list);
    }
    /**
     * 新增计量单位
     */
    @ApiOperation("新增计量单位")
    @Log(title = "计量单位", businessType = BusinessType.INSERT)
    @PostMapping
    @Transactional
    public AjaxResult add(@RequestBody List<BizMeasurement> bizMeasurementList) throws NoSuchFieldException, IllegalAccessException
    {
        //判断传入的个数
        if(bizMeasurementList.size() == 0){
            throw new RuntimeException("新增失败,数据不能为空");
        }
        if(bizMeasurementList.size() >100) {
            throw new RuntimeException("新增失败,数据过多,不允许超过100条");
        }
            if (bizMeasurementList.stream().anyMatch(measurement -> measurement.getCode() == null || measurement.getCode() == "")) {
                throw new RuntimeException("新增失败,编码不能为空");
            }
            //判断编码是否有重复
            List<String> measurementCodes = bizMeasurementList.stream()
                    .map(BizMeasurement::getCode)
                    .collect(Collectors.toList());
            Set<String> uniqueMeasurementCodes = new HashSet<>(measurementCodes);
            if (measurementCodes.size() != uniqueMeasurementCodes.size()) {
                throw new RuntimeException("新增失败,编码不能重复");
        }
                //新增的编码没有重复
                //判断数据库是否存在相同的编码
                List<BizMeasurement> measurements = bizMeasurementService.query().in("code", measurementCodes).list();
                if (!measurements.isEmpty()){
                    throw new RuntimeException("新增失败,存在重复的编码");
                }
        return toAjax(bizMeasurementService.saveBatch(bizMeasurementList));
    }

    /**
     * 修改计量单位
     */
    @ApiOperation("修改计量单位")
    @Log(title = "计量单位", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody List<BizMeasurement> bizMeasurementList) throws NoSuchFieldException, IllegalAccessException
    {
        //判断传入的个数
        if(bizMeasurementList.size() == 0){
            throw new RuntimeException("修改失败,数据不能为空");
        }
        if(bizMeasurementList.size() >100) {
            throw new RuntimeException("修改失败,数据过多,不允许超过100条");
        }
        if (bizMeasurementList.stream().anyMatch(measurement -> measurement.getHistoryCode() == null ||measurement.getHistoryCode() == "")) {
            throw new RuntimeException("修改失败,原始编码不能为空");
        }
        //判断产品编码是否有重复(可能存在相同的id和编码写了多次
        Map<String, List<BizMeasurement>> measurementCodeMap = bizMeasurementList.stream()
                .filter(material -> material.getCode() != null)
                .collect(Collectors.groupingBy(BizMeasurement::getCode));

        boolean hasDuplicate = measurementCodeMap.values().stream()
                .anyMatch(list -> list.size() > 1 &&
                        list.stream().map(BizMeasurement::getHistoryCode).distinct().count() > 1);

        if (hasDuplicate) {
            throw new RuntimeException("修改失败，编码不能重复");
        }
        //判断数据库是否有重复的编码
        List<String> measurementCodes = bizMeasurementList.stream()
                .filter(material -> material.getCode() != null)
                .map(BizMeasurement::getCode)
                .collect(Collectors.toList());
        if (!measurementCodes.isEmpty()) {
            List<BizMeasurement> measurementList = bizMeasurementService.query().in("code", measurementCodes).list();
            for (BizMeasurement measurement : bizMeasurementList) {
                for (BizMeasurement measure : measurementList) {
                    if (Objects.equals(measurement.getCode(), measure.getCode()) &&
                            !Objects.equals(measurement.getHistoryCode(), measure.getCode())) {
                        // 如果找到匹配的项，则抛出错误
                        throw new RuntimeException("修改失败,编码不能重复");
                    }
                }
            }
        }
        List<BizMeasurement> measurementList = bizMeasurementService.query().in("code", bizMeasurementList.stream()
                .map(BizMeasurement::getHistoryCode)
                .collect(Collectors.toList())).list();
        for (BizMeasurement measure : bizMeasurementList) {
            BizMeasurement bizMeasurement =  measurementList.stream().filter(item -> item.getCode().equals(measure.getHistoryCode())).findFirst().orElse(null);
            if (bizMeasurement == null) {
                throw new RuntimeException("修改失败,编码不存在");
            }
            if (measure.getCode() != null) {
                bizMeasurement.setCode(measure.getCode());
            }
            if (measure.getHistoryCode() != null) {
                bizMeasurement.setHistoryCode(measure.getHistoryCode());
            }
            if (measure.getNumerator() != null) {
                bizMeasurement.setNumerator(measure.getNumerator());
            }
            if (measure.getDenominator() != null) {
                bizMeasurement.setDenominator(measure.getDenominator());
            }
            if (measure.getName() != null) {
                bizMeasurement.setName(measure.getName());
            }
            if (measure.getRemake() != null) {
                bizMeasurement.setRemake(measure.getRemake());
            }
            if (measure.getGroup() != null) {
                bizMeasurement.setGroup(measure.getGroup());
            }
            if (measure.getStandard() != null) {
                bizMeasurement.setStandard(measure.getStandard());
            }
            if (measure.getConversionType() != null) {
                bizMeasurement.setConversionType(measure.getConversionType());
            }
            if (measure.getPrecision() != null) {
                bizMeasurement.setPrecision(measure.getPrecision());
            }
            if (measure.getRoundingType() != null) {
                bizMeasurement.setRoundingType(measure.getRoundingType());
            }
            if (measure.getStatus() != null) {
                bizMeasurement.setStatus(measure.getStatus());
            }
            if (measure.getForbidden() != null) {
                bizMeasurement.setForbidden(measure.getForbidden());
            }
            if (measure.getCreateName() != null) {
                bizMeasurement.setCreateName(measure.getCreateName());
            }
            if (measure.getUpdateName() != null) {
                bizMeasurement.setUpdateName(measure.getUpdateName());
            }
            if (measure.getAuditTime() != null) {
                bizMeasurement.setAuditTime(measure.getAuditTime());
            }
            if (measure.getAuditName() != null) {
                bizMeasurement.setAuditName(measure.getAuditName());
            }
            if (measure.getForbiddenTime() != null) {
                bizMeasurement.setForbiddenTime(measure.getForbiddenTime());
            }
            if (measure.getForbiddenName() != null) {
                bizMeasurement.setForbiddenName(measure.getForbiddenName());
            }
            if (measure.getInit() != null) {
                bizMeasurement.setInit(measure.getInit());
            }

        }
            return toAjax(bizMeasurementService.updateBatchById(measurementList));
    }

    /**
     * 删除计量单位
     */
    @ApiOperation("删除计量单位")
    @Log(title = "计量单位", businessType = BusinessType.DELETE)
	@DeleteMapping("/{codes}")
    public AjaxResult remove(@PathVariable List<String> codes)
    {
        if(codes.isEmpty()){
            return error("删除失败，编码不能为空");
        }
        List<BizMeasurement> list = bizMeasurementService.query().in("code", codes).list();
        return toAjax(bizMeasurementService.removeBatchByIds(list));
    }
}
